"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.searchEntitiesIds = exports.searchEvents = void 0;
const constants_1 = require("../constants");
const keys_helper_1 = require("./keys-helper");
const pagination_helpers_1 = require("../pagination-helpers");
const framework_common_helpers_1 = require("@boostercloud/framework-common-helpers");
async function searchEvents(dynamoDB, config, parameters) {
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'events-searcher-adapter#searchEvents');
    logger.debug('Initiating an events search. Filters: ', parameters);
    const timeFilterQuery = buildSearchEventsTimeQuery(parameters.from, parameters.to);
    const eventEnvelopes = await executeSearch(dynamoDB, config, parameters, timeFilterQuery, parameters.limit);
    logger.debug('Events search result: ', eventEnvelopes);
    return convertToSearchResult(eventEnvelopes);
}
exports.searchEvents = searchEvents;
async function searchEntitiesIds(dynamoDB, config, limit, afterCursor, entityTypeName) {
    throw new Error('eventsSearcherAdapter#searchEntitiesIds: Not implemented yet');
}
exports.searchEntitiesIds = searchEntitiesIds;
function buildSearchEventsTimeQuery(from, to) {
    let timeQueryData = {
        expression: '',
        attributeValues: {},
    };
    if (from && to) {
        timeQueryData = {
            expression: ` AND ${constants_1.eventsStoreAttributes.sortKey} BETWEEN :fromTime AND :toTime`,
            attributeValues: {
                ':fromTime': from,
                ':toTime': to,
            },
        };
    }
    else if (from) {
        timeQueryData = {
            expression: ` AND ${constants_1.eventsStoreAttributes.sortKey} >= :fromTime`,
            attributeValues: { ':fromTime': from },
        };
    }
    else if (to) {
        timeQueryData = {
            expression: ` AND ${constants_1.eventsStoreAttributes.sortKey} <= :toTime`,
            attributeValues: { ':toTime': to },
        };
    }
    return timeQueryData;
}
async function executeSearch(dynamoDB, config, filters, timeFilterQuery, limit) {
    if ('entity' in filters) {
        if (filters.entityID) {
            return await searchEventsByEntityAndID(dynamoDB, config, filters.entity, filters.entityID, timeFilterQuery, limit);
        }
        else {
            return await searchEventsByEntity(dynamoDB, config, filters.entity, timeFilterQuery, limit);
        }
    }
    else if ('type' in filters) {
        return await searchEventsByType(dynamoDB, config, filters.type, timeFilterQuery, limit);
    }
    else {
        throw new Error('Invalid search event query. It is neither an search by "entity" nor a search by "type"');
    }
}
async function searchEventsByEntityAndID(dynamoDB, config, entity, entityID, timeQuery, limit) {
    var _a;
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'events-searcher-adapter#searchEventsByEntityAndID');
    // TODO: Manage pagination
    const params = {
        TableName: config.resourceNames.eventsStore,
        ConsistentRead: true,
        ScanIndexForward: false,
        Limit: limit,
        KeyConditionExpression: `${constants_1.eventsStoreAttributes.partitionKey} = :partitionKey${timeQuery.expression}`,
        ExpressionAttributeValues: {
            ...timeQuery.attributeValues,
            ':partitionKey': (0, keys_helper_1.partitionKeyForEvent)(entity, entityID),
        },
    };
    logger.debug('Searching events by entity and entity ID. Query params: ', params);
    const result = await dynamoDB.query(params).promise();
    return (_a = result.Items) !== null && _a !== void 0 ? _a : [];
}
async function searchEventsByEntity(dynamoDB, config, entity, timeQuery, limit) {
    var _a;
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'events-searcher-adapter#searchEventsByEntity');
    // TODO: manage pagination
    // First query the index
    const params = {
        TableName: config.resourceNames.eventsStore,
        IndexName: constants_1.eventsStoreAttributes.indexByEntity.name(config),
        ScanIndexForward: false,
        Limit: limit,
        KeyConditionExpression: `${constants_1.eventsStoreAttributes.indexByEntity.partitionKey} = :partitionKey${timeQuery.expression}`,
        ExpressionAttributeValues: {
            ...timeQuery.attributeValues,
            ':partitionKey': (0, keys_helper_1.partitionKeyForIndexByEntity)(entity, 'event'),
        },
    };
    logger.debug('Searching events by entity. Index query params: ', params);
    const partialResult = await dynamoDB.query(params).promise();
    const indexRecords = (_a = partialResult.Items) !== null && _a !== void 0 ? _a : [];
    return findEventsDataWithKeys(dynamoDB, config, indexRecords);
}
async function searchEventsByType(dynamoDB, config, type, timeQuery, limit) {
    var _a;
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'events-searcher-adapter#searchEventsByType');
    // TODO: manage pagination
    // Fist query the index
    const params = {
        TableName: config.resourceNames.eventsStore,
        IndexName: constants_1.eventsStoreAttributes.indexByType.name(config),
        ScanIndexForward: false,
        Limit: limit,
        KeyConditionExpression: `${constants_1.eventsStoreAttributes.indexByType.partitionKey} = :partitionKey${timeQuery.expression}`,
        ExpressionAttributeValues: {
            ...timeQuery.attributeValues,
            ':partitionKey': type,
        },
    };
    logger.debug('Searching events by type. Index query params: ', params);
    const partialResult = await dynamoDB.query(params).promise();
    const indexRecords = (_a = partialResult.Items) !== null && _a !== void 0 ? _a : [];
    return findEventsDataWithKeys(dynamoDB, config, indexRecords);
}
async function findEventsDataWithKeys(dynamoDB, config, keys) {
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'events-searcher-adapter#findEventsDataWithKeys');
    const result = [];
    const keysBatches = (0, pagination_helpers_1.inChunksOf)(constants_1.dynamoDbBatchGetLimit, keys);
    logger.debug(`Performing batch get for ${keysBatches.length} batches`);
    for (const keysBatch of keysBatches) {
        const batchResult = await performBatchGet(dynamoDB, config, keysBatch);
        result.push(...batchResult);
    }
    return result;
}
async function performBatchGet(dynamoDB, config, keys) {
    var _a, _b;
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'events-searcher-adapter#performBatchGet');
    const params = {
        RequestItems: {
            [config.resourceNames.eventsStore]: {
                ConsistentRead: true,
                Keys: keys.map((record) => {
                    return {
                        [constants_1.eventsStoreAttributes.partitionKey]: record[constants_1.eventsStoreAttributes.partitionKey],
                        [constants_1.eventsStoreAttributes.sortKey]: record[constants_1.eventsStoreAttributes.sortKey],
                    };
                }),
            },
        },
    };
    logger.debug('Finding events data for keys: ', keys, params);
    const result = await dynamoDB.batchGet(params).promise();
    return (_b = (_a = result.Responses) === null || _a === void 0 ? void 0 : _a[config.resourceNames.eventsStore]) !== null && _b !== void 0 ? _b : [];
}
function convertToSearchResult(eventEnvelopes) {
    // "eventEnvelopes" represent a result page. That page is absolutely ordered among all the items in the
    // database. BUT! the local order within the page is been messed up, so we need to sort the items.
    // Why the order is messed up? Because we are doing two queries to DynamoDB:
    // - One to the corresponding table index to know the keys of the items we need to retrieve from the master table.
    // The result of this query is paginated and the absolute order of items is respected.
    // - Another one to the master table to get the items data. This query is made with "batchQueryItems", which
    // does not preserve the order in which we specify the keys. This is why we need to sort the final result.
    // It affects also to the limit that could only be applied to the ordered elements
    return eventEnvelopes
        .map((eventEnvelope) => {
        return {
            type: eventEnvelope.typeName,
            entity: eventEnvelope.entityTypeName,
            entityID: eventEnvelope.entityID,
            requestID: eventEnvelope.requestID,
            user: eventEnvelope.currentUser,
            createdAt: eventEnvelope.createdAt,
            value: eventEnvelope.value,
            deletedAt: eventEnvelope.deletedAt,
        };
    })
        .sort((a, b) => {
        if (a.createdAt > b.createdAt)
            return -1;
        if (a.createdAt < b.createdAt)
            return 1;
        return 0;
    });
}
