"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteReadModel = exports.storeReadModel = exports.fetchReadModel = exports.rawReadModelEventsToEnvelopes = void 0;
/* eslint-disable @typescript-eslint/no-explicit-any,@typescript-eslint/no-magic-numbers */
const framework_types_1 = require("@boostercloud/framework-types");
const dynamodb_1 = require("aws-sdk/clients/dynamodb");
const arn_1 = require("./arn");
const framework_common_helpers_1 = require("@boostercloud/framework-common-helpers");
async function rawReadModelEventsToEnvelopes(config, rawEvents) {
    return rawEvents.Records.map(toReadModelEnvelope.bind(null, config));
}
exports.rawReadModelEventsToEnvelopes = rawReadModelEventsToEnvelopes;
async function fetchReadModel(db, config, readModelName, readModelID, sequenceKey) {
    var _a;
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'read-models-adapter#fetchReadModel');
    // We're using query for get single read models too as the difference in performance
    // between get-item and query is none for a single item.
    // Source: https://forums.aws.amazon.com/thread.jspa?threadID=93743
    const sequenceKeyCondition = (sequenceKey === null || sequenceKey === void 0 ? void 0 : sequenceKey.value) ? ` AND #${sequenceKey.name} = :${sequenceKey.name}` : '';
    const sequenceAttributeNames = (sequenceKey === null || sequenceKey === void 0 ? void 0 : sequenceKey.value) ? { [`#${sequenceKey.name}`]: sequenceKey.name } : {};
    const sequenceAttributeValues = (sequenceKey === null || sequenceKey === void 0 ? void 0 : sequenceKey.value) ? { [`:${sequenceKey.name}`]: sequenceKey.value } : {};
    logger.debug(`[ReadModelAdapter#fetchReadModel] Performing query for ${readModelName}, with ID ${readModelID} and sequenceKey`, sequenceKey);
    const queryParams = {
        TableName: config.resourceNames.forReadModel(readModelName),
        KeyConditionExpression: '#id = :id' + sequenceKeyCondition,
        ExpressionAttributeNames: {
            '#id': 'id',
            ...sequenceAttributeNames,
        },
        ExpressionAttributeValues: {
            ':id': readModelID,
            ...sequenceAttributeValues,
        },
        /*
         * TODO: We need to have consistent reads active here to manage the case when we write a new version of the read model
         * and read it in the same lambda execution. For instance, when we're processing the event stream and updating the entities,
         * then read models are updated. We need to make sure that the second time we read it we get the same object we stored in
         * the previous iteration.
         *
         * Still, it would be interesting to make eventual consistent reads for requests coming from the API, as they're faster.
         * One possible solution would be having an extra optional parameter that defaults to `true`, but can be set to `false` in
         * that scenario.
         */
        ConsistentRead: true,
    };
    const response = await db.query(queryParams).promise();
    logger.debug(`[ReadModelAdapter#fetchReadModel] Loaded read model ${readModelName} with ID ${readModelID} with result:`, response.Items);
    if (!response.Items || ((_a = response.Items) === null || _a === void 0 ? void 0 : _a.length) === 0) {
        return response.Items;
    }
    return response.Items.map((value) => {
        const optimisticConcurrencyValue = generateOptimisticConcurrencyValue(value);
        return {
            ...value,
            boosterMetadata: {
                ...value === null || value === void 0 ? void 0 : value.boosterMetadata,
                optimisticConcurrencyValue,
            },
        };
    });
}
exports.fetchReadModel = fetchReadModel;
async function storeReadModel(db, config, readModelName, readModel, expectedCurrentVersion) {
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'read-models-adapter#storeReadModel');
    try {
        await db
            .put({
            TableName: config.resourceNames.forReadModel(readModelName),
            Item: readModel,
            ConditionExpression: 'attribute_not_exists(boosterMetadata.optimisticConcurrencyValue) OR boosterMetadata.optimisticConcurrencyValue = :optimisticConcurrencyValue',
            ExpressionAttributeValues: { ':optimisticConcurrencyValue': expectedCurrentVersion },
        })
            .promise();
    }
    catch (e) {
        const error = e;
        // The error will be thrown, but in case of a conditional check, we throw the expected error type by the core
        if (error.name == 'ConditionalCheckFailedException') {
            throw new framework_types_1.OptimisticConcurrencyUnexpectedVersionError(error.message);
        }
        throw e;
    }
    logger.debug('[ReadModelAdapter#storeReadModel] Read model stored');
}
exports.storeReadModel = storeReadModel;
async function deleteReadModel(db, config, readModelName, readModel) {
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'read-models-adapter#deleteReadModel');
    const sequenceKeyName = config.readModelSequenceKeys[readModelName];
    await db
        .delete({
        TableName: config.resourceNames.forReadModel(readModelName),
        Key: buildKey(readModel.id, sequenceKeyName, readModel[sequenceKeyName]),
    })
        .promise();
    logger.debug(`[ReadModelAdapter#deleteReadModel] Read model deleted. ID = ${readModel.id}`);
}
exports.deleteReadModel = deleteReadModel;
function toReadModelEnvelope(config, record) {
    var _a;
    if (!((_a = record.dynamodb) === null || _a === void 0 ? void 0 : _a.NewImage) || !record.eventSourceARN) {
        throw new Error('Received a DynamoDB stream event without "eventSourceARN" or "NewImage" field. They are required');
    }
    const tableARNComponents = arn_1.Arn.parse(record.eventSourceARN);
    if (!tableARNComponents.resourceName) {
        throw new Error('Could not extract the table name from the eventSourceARN');
    }
    const readModelTableName = tableARNComponents.resourceName.split('/')[0];
    return {
        typeName: config.readModelNameFromResourceName(readModelTableName),
        value: dynamodb_1.Converter.unmarshall(record.dynamodb.NewImage),
    };
}
function buildKey(readModelID, sequenceKeyName, sequenceKeyValue) {
    if (sequenceKeyName && sequenceKeyValue)
        return { id: readModelID, [sequenceKeyName]: sequenceKeyValue };
    else
        return { id: readModelID };
}
function generateOptimisticConcurrencyValue(value) {
    var _a, _b;
    return ((_b = (_a = value.boosterMetadata) === null || _a === void 0 ? void 0 : _a.version) !== null && _b !== void 0 ? _b : 0) + 1; // the next version number that we are going to persist
}
