"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.searchReadModel = void 0;
/* eslint-disable @typescript-eslint/no-explicit-any */
const framework_types_1 = require("@boostercloud/framework-types");
const framework_common_helpers_1 = require("@boostercloud/framework-common-helpers");
async function searchReadModel(dynamoDB, config, readModelName, filters, sortBy, limit, afterCursor, paginatedVersion = false) {
    var _a, _b;
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'read-model-searcher-adapter#searchReadModel');
    if (sortBy) {
        logger.info('SortBy not implemented for AWS provider. It will be ignored');
    }
    let params = {
        TableName: config.resourceNames.forReadModel(readModelName),
        ConsistentRead: true,
        Limit: limit,
        ExclusiveStartKey: afterCursor,
    };
    if (filters && Object.keys(filters).length > 0) {
        params = {
            ...params,
            FilterExpression: buildFilterExpression(filters),
            ExpressionAttributeNames: buildExpressionAttributeNames(filters),
        };
        const expressionAttributeValues = buildExpressionAttributeValues(filters);
        if (expressionAttributeValues && Object.keys(expressionAttributeValues).length > 0) {
            params = {
                ...params,
                ExpressionAttributeValues: expressionAttributeValues,
            };
        }
    }
    logger.debug('Running search with the following params: \n', params);
    const result = await dynamoDB.scan(params).promise();
    logger.debug('Search result: ', result);
    if (paginatedVersion) {
        return {
            items: (_a = result.Items) !== null && _a !== void 0 ? _a : [],
            count: result.Count,
            cursor: result.LastEvaluatedKey,
        };
    }
    else {
        return (_b = result.Items) !== null && _b !== void 0 ? _b : [];
    }
}
exports.searchReadModel = searchReadModel;
function buildFilterExpression(filters, usedPlaceholders = []) {
    return Object.entries(filters)
        .map(([propName, filter]) => {
        switch (propName) {
            case 'not':
                return `NOT (${buildFilterExpression(filter, usedPlaceholders)})`;
            case 'and':
            case 'or':
                return `(${filter
                    .map((arrayFilter) => buildFilterExpression(arrayFilter, usedPlaceholders))
                    .join(` ${propName} `)})`;
            default:
                return buildOperation(propName, filter, usedPlaceholders);
        }
    })
        .join(' AND ');
}
function buildOperation(propName, filter = {}, usedPlaceholders, nested) {
    const holder = placeholderBuilderFor(propName, usedPlaceholders);
    propName = nested ? `${nested}.#${propName}` : propName;
    return Object.entries(filter)
        .map(([operation, value], index) => {
        switch (operation) {
            case 'eq':
                return `#${propName} = ${holder(index)}`;
            case 'ne':
                return `#${propName} <> ${holder(index)}`;
            case 'lt':
                return `#${propName} < ${holder(index)}`;
            case 'gt':
                return `#${propName} > ${holder(index)}`;
            case 'gte':
                return `#${propName} >= ${holder(index)}`;
            case 'lte':
                return `#${propName} <= ${holder(index)}`;
            case 'in':
                return `#${propName} IN (${value
                    .map((value, subIndex) => `${holder(index, subIndex)}`)
                    .join(',')})`;
            case 'contains':
                return `contains(#${propName}, ${holder(index)})`;
            case 'beginsWith':
                return `begins_with(#${propName}, ${holder(index)})`;
            case 'includes':
                return `contains(#${propName}, ${holder(index)})`;
            case 'isDefined':
                if (value) {
                    return `attribute_exists(#${propName})`;
                }
                else {
                    return `attribute_not_exists(#${propName})`;
                }
            default:
                if (typeof value === 'object') {
                    return buildOperation(operation, value, usedPlaceholders, propName);
                }
                throw new framework_types_1.InvalidParameterError(`Operator "${operation}" is not supported`);
        }
    })
        .join(' AND ');
}
function placeholderBuilderFor(propName, usedPlaceholders) {
    return (valueIndex, valueSubIndex) => {
        const placeholder = `:${propName}_${valueIndex}` + (typeof valueSubIndex === 'number' ? `_${valueSubIndex}` : '');
        if (usedPlaceholders.includes(placeholder))
            return placeholderBuilderFor(propName, usedPlaceholders)(valueIndex + 1);
        usedPlaceholders.push(placeholder);
        return placeholder;
    };
}
function buildExpressionAttributeNames(filters) {
    const attributeNames = {};
    for (const propName in filters) {
        switch (propName) {
            case 'not':
                Object.assign(attributeNames, buildExpressionAttributeNames(filters[propName]));
                break;
            case 'and':
            case 'or':
                for (const filter of filters[propName]) {
                    Object.assign(attributeNames, buildExpressionAttributeNames(filter));
                }
                break;
            case 'includes':
            case 'isDefined':
                // Avoid the default behaviour
                break;
            default:
                Object.entries(filters[propName]).forEach(([prop, value]) => {
                    attributeNames[`#${propName}`] = propName;
                    if (typeof value === 'object' && !Array.isArray(value) && value !== null) {
                        Object.assign(attributeNames, buildExpressionAttributeNames({ [prop]: value }));
                    }
                });
                break;
        }
    }
    return attributeNames;
}
function buildExpressionAttributeValues(filters, usedPlaceholders = []) {
    const attributeValues = {};
    Object.entries(filters).forEach(([propName]) => {
        switch (propName) {
            case 'not':
                Object.assign(attributeValues, buildExpressionAttributeValues(filters[propName], usedPlaceholders));
                break;
            case 'and':
            case 'or':
                for (const filter of filters[propName]) {
                    Object.assign(attributeValues, buildExpressionAttributeValues(filter, usedPlaceholders));
                }
                break;
            default:
                Object.assign(attributeValues, buildAttributeValue(propName, filters[propName], usedPlaceholders));
                break;
        }
    });
    return attributeValues;
}
function buildAttributeValue(propName, filter = {}, usedPlaceholders) {
    const attributeValues = {};
    const holder = placeholderBuilderFor(propName, usedPlaceholders);
    Object.entries(filter).forEach(([key, value], index) => {
        if (Array.isArray(value)) {
            value.forEach((element, subIndex) => {
                attributeValues[holder(index, subIndex)] = element;
            });
        }
        else if (typeof value === 'object' && key !== 'includes' && value !== null) {
            Object.assign(attributeValues, buildExpressionAttributeValues({ [key]: value }, usedPlaceholders));
        }
        else if (key !== 'isDefined') {
            attributeValues[holder(index)] = value;
        }
    });
    return attributeValues;
}
