"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteAllSubscriptions = exports.deleteSubscription = exports.fetchSubscriptions = exports.subscribeToReadModel = void 0;
const constants_1 = require("../constants");
const keys_helper_1 = require("./keys-helper");
const framework_common_helpers_1 = require("@boostercloud/framework-common-helpers");
async function subscribeToReadModel(db, config, subscriptionEnvelope) {
    if (!subscriptionEnvelope[constants_1.subscriptionsStoreAttributes.partitionKey] ||
        !subscriptionEnvelope.connectionID || // First part of subscriptionsStoreAttributes.sortKey
        !subscriptionEnvelope.operation.id || // Second part of subscriptionsStoreAttributes.sortKey
        !subscriptionEnvelope[constants_1.subscriptionsStoreAttributes.ttl]) {
        throw new Error('Subscription envelope is missing any of the following required attributes: ' +
            `"${constants_1.subscriptionsStoreAttributes.partitionKey}", "connectionID", "operation.id", ${constants_1.subscriptionsStoreAttributes.ttl}"`);
    }
    await db
        .put({
        TableName: config.resourceNames.subscriptionsStore,
        Item: {
            ...subscriptionEnvelope,
            [constants_1.subscriptionsStoreAttributes.sortKey]: (0, keys_helper_1.sortKeyForSubscription)(subscriptionEnvelope.connectionID, subscriptionEnvelope.operation.id),
            [constants_1.subscriptionsStoreAttributes.indexByConnectionIDSortKey]: subscriptionEnvelope.operation.id,
        },
    })
        .promise();
}
exports.subscribeToReadModel = subscribeToReadModel;
async function fetchSubscriptions(db, config, subscriptionName) {
    // TODO: filter expired ones. Or... is it needed?
    const result = await db
        .query({
        TableName: config.resourceNames.subscriptionsStore,
        ConsistentRead: true,
        KeyConditionExpression: `${constants_1.subscriptionsStoreAttributes.partitionKey} = :partitionKey`,
        ExpressionAttributeValues: {
            ':partitionKey': subscriptionName,
        },
    })
        .promise();
    return result.Items;
}
exports.fetchSubscriptions = fetchSubscriptions;
async function deleteSubscription(db, config, connectionID, subscriptionID) {
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'subscription-adapter#deleteSubscription');
    // TODO: Manage query pagination
    const result = await db
        .query({
        TableName: config.resourceNames.subscriptionsStore,
        IndexName: constants_1.subscriptionsStoreAttributes.indexByConnectionIDName(config),
        KeyConditionExpression: `${constants_1.subscriptionsStoreAttributes.indexByConnectionIDPartitionKey} = :partitionKey AND ` +
            `${constants_1.subscriptionsStoreAttributes.indexByConnectionIDSortKey} = :sortKey`,
        ExpressionAttributeValues: {
            ':partitionKey': connectionID,
            ':sortKey': subscriptionID,
        },
    })
        .promise();
    const foundSubscriptions = result.Items;
    if ((foundSubscriptions === null || foundSubscriptions === void 0 ? void 0 : foundSubscriptions.length) < 1) {
        logger.info(`[deleteSubscription] No subscriptions found with connectionID=${connectionID} and subscriptionID=${subscriptionID}`);
        return;
    }
    const subscriptionToDelete = foundSubscriptions[0]; // There can't be more than one, as we used the full primary key in the query
    logger.debug('[deleteSubscription] Deleting subscription = ', subscriptionToDelete);
    await db
        .delete({
        TableName: config.resourceNames.subscriptionsStore,
        Key: {
            [constants_1.subscriptionsStoreAttributes.partitionKey]: subscriptionToDelete[constants_1.subscriptionsStoreAttributes.partitionKey],
            [constants_1.subscriptionsStoreAttributes.sortKey]: subscriptionToDelete[constants_1.subscriptionsStoreAttributes.sortKey],
        },
    })
        .promise();
}
exports.deleteSubscription = deleteSubscription;
async function deleteAllSubscriptions(db, config, connectionID) {
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'subscription-adapter#deleteAllSubscriptions');
    // TODO: Manage query pagination and db.batchWrite limit of 25 operations at a time
    const result = await db
        .query({
        TableName: config.resourceNames.subscriptionsStore,
        IndexName: constants_1.subscriptionsStoreAttributes.indexByConnectionIDName(config),
        KeyConditionExpression: `${constants_1.subscriptionsStoreAttributes.indexByConnectionIDPartitionKey} = :partitionKey`,
        ExpressionAttributeValues: { ':partitionKey': connectionID },
    })
        .promise();
    const foundSubscriptions = result.Items;
    if ((foundSubscriptions === null || foundSubscriptions === void 0 ? void 0 : foundSubscriptions.length) < 1) {
        logger.info(`[deleteAllSubscription] No subscriptions found with connectionID=${connectionID}`);
        return;
    }
    logger.debug(`[deleteAllSubscription] Deleting all subscriptions for connectionID=${connectionID}, which are: `, foundSubscriptions);
    const params = {
        RequestItems: {
            [config.resourceNames.subscriptionsStore]: foundSubscriptions.map((subscriptionRecord) => ({
                DeleteRequest: {
                    Key: {
                        [constants_1.subscriptionsStoreAttributes.partitionKey]: subscriptionRecord[constants_1.subscriptionsStoreAttributes.partitionKey],
                        [constants_1.subscriptionsStoreAttributes.sortKey]: subscriptionRecord[constants_1.subscriptionsStoreAttributes.sortKey],
                    },
                },
            })),
        },
    };
    await db.batchWrite(params).promise();
}
exports.deleteAllSubscriptions = deleteAllSubscriptions;
